<!-- 
This README describes the package. If you publish this package to pub.dev,
this README's contents appear on the landing page for your package.

For information about how to write a good package README, see the guide for
[writing package pages](https://dart.dev/guides/libraries/writing-package-pages). 

For general information about developing packages, see the Dart guide for
[creating packages](https://dart.dev/guides/libraries/create-library-packages)
and the Flutter guide for
[developing packages and plugins](https://flutter.dev/developing-packages). 
-->

Use a `FirestoreRepository<TModel>` where `TModel` can be anything.
The Repository needs a `JsonSerializer<TModel>` to convert between `TModel` and Firestore object.
You must also provide a `collectionPath` to the repository.

<!--
## Features

TODO: List what your package can do. Maybe include images, gifs, or videos.

## Getting started

TODO: List prerequisites and provide or point to information on how to
start using the package.
-->
## Usage

Creating the repository for a specific model

```dart
class MyModel implements FirestoreObject<MyModel> {
  String id;
  final String myProperty;

  MyModel({required this.id, required this.myProperty});

  @override
  set(String id) => this.id = id;
}

class MyModelSerializer implements JsonSerializer<MyModel> {
  @override
  MyModel fromJson(Map<String, dynamic> json) => MyModel(id: "", myProperty: json["myProperty"]);

  @override
  Map<String, dynamic> toJson(MyModel data) => {"myProperty": data.myProperty};
}

class MyModelFirestoreRepository extends FirestoreRepositoryImpl<MyModel>{
  @override
  String get collectionPath => "my_custom_model_collection";

  @override
  JsonSerializer<MyModel> get serializer => MyModelSerializer();
}
```

Using the repository
````dart
final myModelRepository = MyModelFirestoreRepository();
  //read
  final allMyModels = await myModelRepository.readAll();
  final singleMyModel = await myModelRepository.readDocument("myFavoriteItem");
  final queriedFavoriteModels = await myModelRepository.query(
      (collection) => collection.where("myProperty", isEqualTo: "favorite"));
  //listen
  final Stream<Resource<MyModel>> listenToDoc =
      myModelRepository.listenToDocument("myObservableItem");
  final Stream<Resource<List<MyModel>>> listenToCollection =
      myModelRepository.listenToAll();
  final Stream<Resource<List<MyModel>>> listenToQuery =
      myModelRepository.listenToQuery((collection) =>
          collection.where("myProperty", isEqualTo: "notFavorite"));

  //write
  await myModelRepository
      .createDocument(MyModel(id: "myItem", myProperty: "favorite"));
  await myModelRepository
      .updateDocument(MyModel(id: "myItem", myProperty: "notFavorite"));
  await myModelRepository.deleteDocument("myItem");
````

## Additional information

TODO: Tell users more about the package: where to find more information, how to 
contribute to the package, how to file issues, what response they can expect 
from the package authors, and more.
